package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import pps.cloud.prediction.service.IThirdActivePowerCloudService;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerInput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerOutput;
import pps.cloud.space.service.IDailyElectricityTrendCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.space.entity.SpaceOptimizeShortDurationView;
import pps.core.space.mapper.SpaceOptimizeShortDurationViewMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 日用电趋势计算Cloud模块
 *
 * @author ZWT
 * @date 2024/05/22 17:17
 */
@XService
public class DailyElectricityTrendCloudServiceImpl extends SpaceOptimizeBaseService implements IDailyElectricityTrendCloudService {

    /**
     * 日用电趋势计算
     * <p>
     * (1)取昨日极短期间开优化结果
     * (2)绿电阶段开井时间：每15分钟判段光伏出力>运行功率（考虑开井顺序,功率累加） 并且当前时段开井，则算坐开井时段(0.25小时)，全天累加
     * (3)单井绿电用电：运行功率*绿电阶段开井时间
     * (4)市电用电:总用电-光伏用电
     *
     * @param context 上下文
     * @return {@link XServiceResult }
     */
    @Override
    public XServiceResult dailyElectricityTrendJob(XContext context) {
        DateTime yesterday = DateUtil.beginOfDay(DateUtil.yesterday());
        //查每15分钟的有功功率
        IThirdActivePowerCloudService service = context.getBean(IThirdActivePowerCloudService.class);
        XListResult<GetThirdActivePowerOutput> result = service.queryAvgPhotovoltaicPower(context, GetThirdActivePowerInput.builder()
                .createDate(yesterday)
                .build());
        result.throwIfFail();
        List<GetThirdActivePowerOutput> powerList = result.getResult();
        if (CollUtil.isEmpty(powerList)) {
            return XServiceResult.OK;
        }
        //查昨日间开优化结果
        SpaceOptimizeShortDurationViewMapper shortMapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
        List<SpaceOptimizeShortDurationView> durationList = shortMapper.selectOptimizationResultList(SpaceOptimizeShortDurationView.builder()
                .optimizeDate(yesterday)
                .build());
        if (CollUtil.isEmpty(durationList)) {
            return XServiceResult.OK;
        }
        //转map
        Map<String, List<GetThirdActivePowerOutput>> powerMap = powerList.stream()
                .collect(Collectors.groupingBy(GetThirdActivePowerOutput::getStationName));
        Map<String, List<SpaceOptimizeShortDurationView>> lineMap = durationList.stream()
                .collect(Collectors.groupingBy(SpaceOptimizeShortDurationView::getLineName));

        List<DateTime> rangeToList = DateUtil.rangeToList(BusinessConstant.DATE_FLAG_BEGIN, BusinessConstant.DATE_FLAG, DateField.MINUTE, 15);
        return null;
    }
}
